
const fs = require('fs');
const execFile = require('child_process').execFile;

// get message(data) from extension
// 拡張からデータをもらう
process.stdin.on('readable', () => {
  var chunks = [];
  var chunk;

  // get message Buffer data
  while (null !== (chunk = process.stdin.read())) {
    chunks.push(chunk);
  }
  chunks = Buffer.concat(chunks);

  var msgLen = chunks.readUInt32LE(0);
  var dataLen = msgLen + 4;

  if (chunks.length >= dataLen) {
    var content = chunks.slice(4, dataLen);
    var json = JSON.parse(content.toString());
    pathControl(json);
  }
});

// determine whether launch Explorer or return native messaging folder path
// Explorerを開くか、ネイティブメッセージングのフォルダパスをレスポンスするか決める
function pathControl(msg) {
  if(msg.todo === "open-explorer") {
    launchExplorer(msg.url);
  } else if( msg.todo === "get-path") {
    responseBack({options:null, cwd:__dirname});
  }
}

// launch file path in Explorer
// クリックしたファイルパスをExplorerで開く
const launchExplorer = (filePath) => {
  filePath = decodeURIComponent(filePath.replace(/#.*/, ''));
  fs.stat(new URL(filePath), (err, stats) => {
    
    // if error, return error response
    // エラーの場合、エラー内容をレスポンスする
    if (err) {
      responseBack({
          filePath,
          resultMessage: 'File Path not Found!',
          err,
      });
      return;
    }

    // if Directory, open Directory Folder in Explorer
    // フォルダパスの場合、フォルダをExplorerで開く
    if (stats.isDirectory()) {
      execFile('explorer', ['/start,', filePath]);
    }
    // if file, open file folder in Explorer
    // ファイルパスの場合、ファイルあるフォルダをExplorerで開く
    else {
      execFile('explorer', ['/select,', filePath]);
    }

    responseBack({
      filePath,
      resultMessage: 'File Path Opended in Explorer',
    });
  });
};

// response message back to Extension background.js
// メッセージを拡張のbackground.jsにレスポンスする
function responseBack(msg) {
  var buffer = Buffer.from(JSON.stringify(msg));
  var header = Buffer.alloc(4);
  header.writeUInt32LE(buffer.length, 0);
  var data = Buffer.concat([header, buffer]);
  process.stdout.write(data);
}

// if error, return error response
// エラーの場合、エラー内容をレスポンスする
process.on('uncaughtException', (err) => {
  responseBack({error: err.toString()});
});